#import "MyColorSelector.h"

/*
Copyright (c) 2001-2008, Maarten Sneep
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the documentation and/or other materials provided with the distribution.
* The names of its contributors may not be used to endorse or promote products derived from this software without specific prior written permission.
THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

@implementation MyColorSelector

- (IBAction)createColor:(id)sender
{
    float r, g, b, c, m, y, k;
    int rh, gh, bh;
    
    [[[colorWell color] colorUsingColorSpaceName:NSCalibratedRGBColorSpace] getRed:&r green:&g blue:&b alpha:NULL];
    
    [LaTeXDefRGB setStringValue:[NSString stringWithFormat:@"\\definecolor{%@}{rgb}{%.3f,%.3f,%.3f}",[colorName stringValue], r, g, b]];

    rh = (int) (r*255.0);
    gh = (int) (g*255.0);
    bh = (int) (b*255.0);
    
    [WebDefRGB setStringValue:[NSString stringWithFormat:@"#%02X%02X%02X; /* %@ */", rh, gh, bh, [colorName stringValue]]];
    
    [[[colorWell color] colorUsingColorSpaceName:NSDeviceCMYKColorSpace] getCyan:&c magenta:&m yellow:&y black:&k alpha:NULL];
    
    [LaTeXDefCMYK setStringValue:[NSString stringWithFormat:@"\\definecolor{%@}{cmyk}{%.3f,%.3f,%.3f,%.3f}",[colorName stringValue], c, m, y, k]];
}

- (IBAction)ingestColor:(id)sender
{
    NSString *str = [sender stringValue];
    NSRange range;
    NSScanner *scanner = [NSScanner scannerWithString:str];
    
    float r, g, b;
    float c, m, y, k;
    unsigned int RGB;
    
    BOOL changedColor = FALSE;
    
    range = [str rangeOfString:@"{rgb}"];
    if (range.location != NSNotFound && range.length > 0) {
        [scanner setCharactersToBeSkipped:[NSCharacterSet characterSetWithCharactersInString:@"{,}"]];
        [scanner setScanLocation:(range.location +  range.length)];
        [scanner scanFloat:&r];
        [scanner scanFloat:&g];
        [scanner scanFloat:&b];
        [colorWell setColor: [NSColor colorWithCalibratedRed:r green:g blue:b alpha:1.00]];
        changedColor = TRUE;
    }
    
    range = [str rangeOfString:@"{cmyk}"];
    if (range.location != NSNotFound && range.length > 0) {
        [scanner setCharactersToBeSkipped:[NSCharacterSet characterSetWithCharactersInString:@"{,}"]];
        [scanner setScanLocation:(range.location +  range.length)];
        
        [scanner scanFloat:&c];
        [scanner scanFloat:&m];
        [scanner scanFloat:&y];
        [scanner scanFloat:&k];
        
        [colorWell setColor: [NSColor colorWithDeviceCyan:c magenta:m yellow:y black:k alpha:1.00]];
        changedColor = TRUE;
    }
    
    range = [str rangeOfString:@"#"];
    if (range.location != NSNotFound && range.length > 0) {
        [scanner setCharactersToBeSkipped:[NSCharacterSet characterSetWithCharactersInString:@"#; "]];
        
        [scanner scanHexInt:&RGB];
        r = ((float) ((RGB & 0x00FF0000)>>16))/255.0;
        g = ((float) ((RGB & 0x0000FF00)>>8))/255.0;
        b = ((float) (RGB & 0x000000FF))/255.0;
        
        [colorWell setColor: [NSColor colorWithCalibratedRed:r green:g blue:b alpha:1.00]];
        changedColor = TRUE;
    }
    
    if (changedColor) {
        [self createColor:self];
    }
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication
{
    return YES;
}

@end
